{% extends "dashboard_app/_content_with_sidebar.html" %}

{% load django_tables2 %}
{% load i18n %}
{% load pagination_tags %}


{% block content %}

{% render_table bundle_stream_table %}

{% endblock %}


{% block sidebar %}
<p>
  {% blocktrans %}
  Streams are containers for bundles and are quite similar to folders or
  directories on your computer.
  {% endblocktrans %}
</p>
<p>{% trans "Currently you have access to:" %}</p>
<ul>
  <li>{% trans "All public streams" %}</li>
  {% if has_personal_streams %}
  <li>{% trans "Your personal streams" %}</li>
  {% endif %}
  {% if has_team_streams %}
  <li>{% trans "Team streams that you are a member of" %}</li>
  {% endif %}
</ul>
{% if not user.is_authenticated %}
<p>{% trans "You must" %} <a href="{% url django.contrib.auth.views.login %}">{% trans "sign in" %}</a> {% trans "to get more access" %}</p>
{% endif %}
{% endblock %}
